#include <fstream>

#include <core/scalar.h>
#include <core/image.h>
#include <core/color.h>
#include <rt/world.h>
#include <rt/renderer.h>
#include <rt/loaders/obj.h>
#include <rt/groups/bvh.h>
#include <rt/groups/simplegroup.h>
#include <rt/groups/bvhio.h>
#include <rt/cameras/perspective.h>
#include <rt/integrators/integrator.h>

using namespace rt;

class TwoSidedRayCastingIntegrator : public Integrator {
public:
    TwoSidedRayCastingIntegrator(World* world)
        : Integrator(world)
    {}

    RGBColor getRadiance(const Ray& ray) const override {
        Intersection hit = world->scene->intersect(ray);
        return RGBColor::rep(hit ? fabsf(dot(hit.normal(), ray.d)) : 0.0f);
    }
};

int main(int argc, char** argv) {
    if (argc < 2)
        return 1;

    Image img(800, 600);

    BVH scene;
    loadOBJ(&scene, "models/", argv[1]);

    std::ifstream is("bvh.bin");
    if (is) {
        BVHStreamInput input(is, scene);
        scene.deserialize(input);
    } else {
        std::ofstream os("bvh.bin");
        BVHStreamOutput output(os, scene);
        scene.rebuildIndex();
        scene.serialize(output);
    }

    World world;
    world.scene = &scene;
    PerspectiveCamera cam(Point(-4.0f, 1.3f, 0.0f), Vector(1.0f, -0.1f, 0.0f), Vector(0, 1, 0), pi/5, pi * (4.0f / 15.0f));
    TwoSidedRayCastingIntegrator integrator(&world);

    Renderer engine(&cam, &integrator);
    engine.render(img);
    img.writePNG("bench.png");
    return 0;
}